const invitationHelper = require('./Invitation')
const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const config = require('config')
const last = require('lodash/last')

const statuses = config.get('statuses')

const addInvitation = async (collection, userId, role) => {
  collection.invitations = collection.invitations || []
  let matchingInvitation = collection.invitations.find(
    invitation =>
      invitation.userId === userId &&
      invitation.role === role &&
      invitation.hasAnswer === false,
  )
  if (matchingInvitation === undefined) {
    matchingInvitation = await invitationHelper.setupInvitation(
      userId,
      role,
      collection,
    )
  } else {
    matchingInvitation.timestamp = Date.now()
  }

  await collection.save()
  return matchingInvitation
}

const addAuthor = async (collection, user, res, url) => {
  if (collection.owners.includes(user.id)) {
    return res.status(200).json(user)
  }
  try {
    await mailService.setupAssignEmail(user.email, 'assign-author', url)

    return res.status(200).json(user)
  } catch (e) {
    logger.error(e)
    return res.status(500).json({ error: 'Email could not be sent.' })
  }
}

const addHandlingEditor = async (collection, user, invitation) => {
  collection.handlingEditor = {
    id: user.id,
    name: `${user.firstName} ${user.lastName}`,
    timestamp: invitation.timestamp,
    email: user.email,
    hasAnswer: invitation.hasAnswer,
    isAccepted: invitation.isAccepted,
  }
  await updateStatus(collection, 'heInvited')
}

const updateHandlingEditor = async (collection, isAccepted) => {
  collection.handlingEditor.hasAnswer = true
  collection.handlingEditor.isAccepted = isAccepted
  collection.handlingEditor.timestamp = Date.now()
  let status
  isAccepted ? (status = 'heAssigned') : (status = 'submitted')
  await updateStatus(collection, status)
}

const getFragmentAndAuthorData = async (models, collection) => {
  const fragment = await models.Fragment.find(last(collection.fragments))
  let { title } = fragment.metadata
  title = title.replace(/<(.|\n)*?>/g, '')

  const submittingAuthorData = collection.authors.find(
    author => author.isSubmitting === true,
  )
  const author = await models.User.find(submittingAuthorData.userId)
  const authorName = `${author.firstName} ${author.lastName}`
  const { id } = fragment
  return { title, authorName, id }
}

const updateReviewerCollectionStatus = async collection => {
  const reviewerInvitations = collection.invitations.filter(
    inv => inv.role === 'reviewer',
  )
  if (reviewerInvitations.length === 0) return
  await updateStatus(collection, 'heAssigned')
}

const updateStatus = async (collection, newStatus) => {
  collection.status = newStatus
  collection.visibleStatus = statuses[collection.status].private
  await collection.save()
}

module.exports = {
  addInvitation,
  addAuthor,
  addHandlingEditor,
  updateHandlingEditor,
  getFragmentAndAuthorData,
  updateReviewerCollectionStatus,
  updateStatus,
}
