const uuid = require('uuid')

const getInvitationData = (invitations, userId, role) => {
  const matchingInvitation = invitations.find(
    inv => inv.role === role && inv.userId === userId,
  )
  let status = 'pending'
  if (matchingInvitation.isAccepted) {
    status = 'accepted'
  } else if (matchingInvitation.hasAnswer) {
    status = 'declined'
  }

  const { timestamp, id } = matchingInvitation
  return { timestamp, status, id }
}

const setupInvitation = async (userId, role, collection) => {
  const invitation = {
    role,
    hasAnswer: false,
    isAccepted: false,
    timestamp: Date.now(),
    id: uuid.v4(),
    userId,
  }
  collection.invitations = collection.invitations || []
  collection.invitations.push(invitation)
  collection = await collection.save()
  return invitation
}

const setupReviewerInvitation = async ({
  baseUrl,
  FragmentModel,
  UserModel,
  collection,
  user,
  mailService,
  invitationId,
  timestamp,
  resend = false,
}) => {
  const fragment = await FragmentModel.find(collection.fragments[0])
  const submittingAuthorData = collection.authors.find(
    author => author.isSubmitting === true,
  )
  const submittingAuthor = await UserModel.find(submittingAuthorData.userId)
  const authorsPromises = collection.authors.map(async author => {
    const user = await UserModel.find(author.userId)
    return `${user.firstName} ${user.lastName}`
  })
  const authors = await Promise.all(authorsPromises)
  let { abstract, title } = fragment.metadata
  title = title.replace(/<(.|\n)*?>/g, '')
  abstract = abstract ? abstract.replace(/<(.|\n)*?>/g, '') : ''

  const params = {
    invitedUser: user,
    baseUrl,
    collection: {
      id: collection.id,
      authorName: `${submittingAuthor.firstName} ${submittingAuthor.lastName}`,
      handlingEditor: collection.handlingEditor,
    },
    subject: `${collection.customId}: Review Requested`,
    fragment: {
      id: fragment.id,
      title,
      abstract,
      authors,
    },
    invitation: {
      id: invitationId,
      timestamp,
    },
    resend,
  }
  await mailService.setupReviewerInvitationEmail(params)
}

const getInvitation = (invitations = [], userId, role) =>
  invitations.find(
    invitation =>
      invitation.userId === userId &&
      invitation.role === role &&
      invitation.hasAnswer === false,
  )

module.exports = {
  getInvitationData,
  setupInvitation,
  setupReviewerInvitation,
  getInvitation,
}
