const helpers = require('./helpers')
const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const collectionHelper = require('./Collection')

const setupNewUser = async (
  body,
  url,
  res,
  email,
  role,
  UserModel,
  invitationType,
) => {
  const { firstName, lastName, affiliation, title } = body
  const newUser = await helpers.createNewUser(
    email,
    firstName,
    lastName,
    affiliation,
    title,
    UserModel,
    role,
  )

  try {
    if (role !== 'reviewer') {
      await mailService.setupInviteEmail(newUser, invitationType, url)
    }

    return newUser
  } catch (e) {
    logger.error(e.message)
    return { status: 500, error: 'Email could not be sent.' }
  }
}

const getEditorInChief = async UserModel => {
  const users = await UserModel.all()
  const eic = users.find(user => user.editorInChief === true)
  return eic
}

const setupReviewerDeclinedEmailData = async (
  models,
  collection,
  req,
  res,
  user,
  mailService,
) => {
  const {
    title,
    authorName,
    id,
  } = await collectionHelper.getFragmentAndAuthorData(models, collection)
  const eic = await getEditorInChief(models.User)
  const toEmail = collection.handlingEditor.email
  try {
    await mailService.setupReviewerDeclinedEmail(
      toEmail,
      user,
      collection,
      title,
      authorName,
      id,
      `${eic.firstName} ${eic.lastName}`,
      `${req.protocol}://${req.get('host')}`,
    )
    await user.save()
    res.status(200).json({})
  } catch (e) {
    logger.error(e)
    return res.status(500).json({ error: 'Email could not be sent.' })
  }
}

const setupReviewerUnassignEmail = async (
  models,
  collection,
  user,
  mailService,
) => {
  const { title, authorName } = await collectionHelper.getFragmentAndAuthorData(
    models,
    collection,
  )

  await mailService.setupReviewerUnassignEmail(
    user,
    collection,
    title,
    authorName,
  )
}

module.exports = {
  setupNewUser,
  getEditorInChief,
  setupReviewerDeclinedEmailData,
  setupReviewerUnassignEmail,
}
